#Anna R. Rogers
#NCSU 2017-2020
#G2F Environmental Data Processing Functions

#Function to move down all date local to a single planting date zero
Plant <- function(data){
  dfTotal <- {} #Set the Total Matrix to being an empty matrix
  maxNumber <- max(data$Number) #Pick out the highest experiment number
  for(i in 1:maxNumber){ #Tells program to do the loop for each group i
    df <- data[data$Number==i,] #Store  the data where Number = i in df
    expMin <- min(df$Day.of.Year) #Pick up the minimum julian date from df
    df$dateSincePlant <- df$Day.of.Year - expMin #For Julian column, subtract the minimum to get the days since planting
    dfTotal <- rbind(dfTotal, df)  #Append this new dataframe onto dfTotal
  }
  return(dfTotal)
}

#Function to take observations from half hour observations to daily observations
DailyMeans <- function(data){
  dfTotal <- {}  #Set the total matrix to being an empty matrix
  maxNumber <- max(data$Number)  #Figure out what the highest number of experiment is
  for(j in 1:maxNumber) { #Tells the loop to be performed for each experiment
    df <- data[data$Number == j, ] #Store the data for experiment j in df
    DailyMean <- {}
    DailyMean$Experiment.s. <- as.character(df$Experiment.s.[1])
    DailyMean$Number <- df$Number[1]
    DailyMean$Station.ID <- df$Station.ID[1]
    MaxDate <- max(df$dateSincePlant)
    #Calculate Daily Means and tell what date it is
    for(i in 0:MaxDate){
      dfa <- df[df$dateSincePlant == i, ] #Store the data from day i of experiment j in dfa
      DailyMean$Day <- dfa$Day[1]
      DailyMean$Month <- dfa$Month[1]
      DailyMean$Year <- dfa$Year[1]
      DailyMean$Day.of.Year <- dfa$Day.of.Year[1]
      
      DailyMean$dateSincePlant <- i
      DailyMean$DailyMeanTemp <- mean(dfa$Temperature..C.[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$DailyHighTemp <- max(dfa$Temperature..C.[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$DailyLowTemp <- min(dfa$Temperature..C.[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$DailyDewPoint <- mean(dfa$Dew.Point..C.[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$DailyRelHumidity <- mean(dfa$Relative.Humidity....[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$DailySolRad <- mean(dfa$Solar.Radiation..W.m2.[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$DailyRain <- mean(dfa$Rainfall..mm.[dfa$dateSincePlant == i], na.rm = TRUE)
      DailyMean$RainTotal <- sum(dfa$Rainfall..mm.[dfa$dateSincePlant == i], na.rm = TRUE)#Do we want total rainfall for each day?
      DailyMean$DailyWind <- mean(dfa$Wind.Speed..m.s.[dfa$dateSincePlant == i], na.rm = TRUE)
  
      dfTotal <- as.data.frame(rbind(dfTotal, DailyMean))  #Append this new dataframe onto dfTotal
    }
  }
  return(dfTotal)
}

#Write a function to find means for each window within an experiment, allowing the user to choose the window size, 
WeatherMeans <- function(data, increment = 30, specified_limit = 151){
  dfTotal <- {} #Set the Total Matrix to being an empty matrix
  maxNumber <- max(data$Number) #Pick out the highest experiment number
  for(i in 1:maxNumber){ #Tells program to do the loop for each group i
    df <- data[data$Number==i,] #Store  the data where Number = i in df
    WM <- {}
    
    #Control Panel
    start <- 1 #Give a starting value for the window
    total_days <- nrow(df)  #See how many total days there are in the weather data for the location
    window_size <- increment + 1  #Set initial window to be the length of the increment + 1 (to account for day zero)
    end <- window_size #Give the initial end
    iteration <- 1
    
    while(end <= specified_limit){
      #print(c(start, window_size, end, iteration))  #Print the start, window size, end, and iteration we are on, as a check to make sure the function is working correctly.
      slice <- df[start:end, ] #Make a slice that includes only the data we want (indexed by start and end values)
      #Calculate values of interest from the slice for the current iteration and store them in a dataframe
      WM$Iteration[iteration] <- iteration
      WM$Experiment[iteration] <- as.character(df$Experiment.s.[1])
      WM$Number[iteration] <- df$Number[1]
      WM$MeanLowTemp[iteration] <- mean(slice$DailyLowTemp, na.rm = TRUE)
      WM$MeanTemp[iteration] <- mean(slice$DailyMeanTemp, na.rm = TRUE)
      WM$MeanHighTemp[iteration] <- mean(slice$DailyHighTemp, na.rm = TRUE)
      WM$MeanDewPoint[iteration] <- mean(slice$DailyDewPoint, na.rm = TRUE)
      WM$MeanHumid[iteration] <- mean(slice$DailyRelHumidity, na.rm = TRUE)
      WM$MeanSol[iteration] <- mean(slice$DailySolRad, na.rm = TRUE)
      WM$MeanRain[iteration] <- mean(slice$RainTotal, na.rm = TRUE)
      WM$MeanWind[iteration] <- mean(slice$DailyWind, na.rm = TRUE)
      WM$RainAccum[iteration] <- sum(slice$RainTotal, na.rm = TRUE)
      WM$PercentDaysWithRain[iteration] <- sum(slice$RainyDayIndicator, na.rm = TRUE)/window_size 
      WM$GDD[iteration] <- sum(slice$GrowingDegreeDays, na.rm = TRUE)
      WM$MeanPhoto[iteration] <- mean(slice$Photoperiod.Hour, na.rm = TRUE)
      WM$MeanGHI[iteration] <- mean(slice$DailyMeanGHI, na.rm = TRUE)
      WM$MeanPeakGHI[iteration] <- mean(slice$DailyPeakGHI, na.rm = TRUE)
      
      #Move to the next iteration of the while loop
      start <- start + window_size #Change the start to the next window
      window_size <- increment  #Change the window size to ensure that it is equal to the increment we want (note that first window is 1 greater to include day zero)
      end <- end + window_size  #Change the window end
      iteration <- iteration + 1  #Up the iteration track by one.
    }
    WM <- as.data.frame(WM) #
      #Use dcast to take the frame from long to wide, for all variables at once
    WM <- dcast(data = setDT(WM), #Set WM to datatable format, 
                  formula = Experiment ~ Iteration, #Use the iteration and Experiment to make the formula, we'll renumber the experiments later anyway so I've dropped the number to make this step easier
                  value.var = c(names(WM)[-c(1:3)])) #Give the variables with values to the formula, so that we expant to wide format
    dfTotal <- as.data.frame(rbind(dfTotal, WM))  #Append this new dataframe onto dfTotal
  }
  return(dfTotal)
}

#Soil data generation
#Triangle lapply adapted from Stack Exchange user Andrew Gustar, bless your soul.
df <- do.call(rbind, lapply(0:100, function(i) data.frame(x=i, y=0:(100-i))))
df$z <- 100-df$x-df$y 

df$sum <- df$x + df$y + df$z
names(df) <- c("Sand", "Silt", "Clay", "Total Composition")

df$`Soil type` <- NA
#Using the clarification of soil texture class boundaries from USDA NSSC NRSC ellis benham
df$`Soil type`[(df$Silt + 1.5*df$Clay) < 15] <- "Sand"
df$`Soil type`[(df$Silt + 1.5*df$Clay) >= 15 & (df$Silt + 2*df$Clay) < 30] <- "Loamy Sand"
df$`Soil type`[((df$Clay >= 7) & (df$Clay < 20) & (df$Sand > 52) & ((df$Silt + 2*df$Clay) >= 30)) | ((df$Clay < 7) & (df$Silt < 50) & ((df$Silt + 2*df$Clay) >= 30))] <- "Sandy Loam"
df$`Soil type`[(df$Clay >= 7) & (df$Clay < 27) & (df$Silt >= 28) & (df$Silt < 50) & (df$Sand <= 52)] <- "Loam"
df$`Soil type`[((df$Silt >= 50) & (df$Clay >= 12) & (df$Clay < 27)) | ((df$Silt >= 50) & (df$Silt < 80) & (df$Clay < 12))] <- "Silt Loam"
df$`Soil type`[(df$Silt >= 80) & (df$Clay < 12)] <- "Silt"
df$`Soil type`[(df$Clay >= 20) & (df$Clay < 35) & (df$Silt < 28) & (df$Sand > 45)] <- "Sandy Clay Loam"
df$`Soil type`[(df$Clay >= 27) & (df$Clay < 40) & (df$Sand > 20) & (df$Sand <= 45)] <- "Clay Loam"
df$`Soil type`[(df$Clay >= 27) & (df$Clay < 40) & (df$Sand <= 20)] <- "Silty Clay Loam"
df$`Soil type`[(df$Clay >= 35) & (df$Sand > 45)] <- "Sandy Clay"
df$`Soil type`[(df$Clay >= 40) & (df$Silt >= 40)] <- "Silty Clay"
df$`Soil type`[(df$Clay >= 40) & (df$Sand <= 45) & (df$Silt < 40)] <- "Clay"

table(df$`Soil type`)
library(ade4)
triangle.plot(df[,1:3]) +
triangle.class(ta = df[, 1:3], fac = as.factor(df$`Soil type`), col = c("#9E0142", "#D53E4F", "#F46D43", "#FDAE61", "#FEE08B", "#FFFFBF", "#E6F598", "#ABDDA4", "#66C2A5", "#3288BD", "#5E4FA2", "#C0C0C0"))

#Make a function to compute soil composition from the ratio of soil types present (2 max in this function)
Soil_Composition <- function(df, Soil_A, Soil_B, Percent_A = .50, Percent_B = .50) {
  temp <- {}
  temp$Sand <- Percent_A*(mean(df$Sand[df$`Soil type` == Soil_A])) + Percent_B*(mean(df$Sand[df$`Soil type` == Soil_B]))
  temp$Silt <- Percent_A*(mean(df$Silt[df$`Soil type` == Soil_A])) + Percent_B*(mean(df$Silt[df$`Soil type` == Soil_B]))
  temp$Clay <- Percent_A*(mean(df$Clay[df$`Soil type` == Soil_A])) + Percent_B*(mean(df$Clay[df$`Soil type` == Soil_B]))
  return(temp)
}